@@ -39,7 +39,7 @@
  *
  */
 
-/* $Id: global.c,v 1.17 2004/11/23 17:40:15 shadow Exp $ */
+/* $Id: global.c,v 1.7 2005/06/17 20:16:29 dasenbro Exp $ */
 
 #include <config.h>
 
@@ -246,9 +246,12 @@ void global_sasl_init(int client, int server, const sasl_callback_t *callbacks)
 	fatal("could not init sasl (client)", EC_SOFTWARE);
     }
 
-    if(server && sasl_server_init(callbacks, "Cyrus")) {
-	fatal("could not init sasl (client)", EC_SOFTWARE);
+    if(server){
+    int r=sasl_server_init_alt(callbacks, "Cyrus");
+    if((r!=SASL_NOMECH)&&(r!=SASL_OK)){
+        fatal("could not init sasl (client)", EC_SOFTWARE);
     }
+    } 
 }
 
 /* this is a wrapper to call the cyrus configuration from SASL */
@@ -605,6 +608,7 @@ void cyrus_ctime(time_t date, char *datebuf)
 	"Jul", "Aug", "Sep", "Oct", "Nov", "Dec" };
 
     if (date == 0 || tm->tm_year < 69) {
+    syslog( LOG_CRIT, "ERROR: invaldi date/time: %d (year = %d)", date, tm->tm_year );
 	abort();
     }
 
